/*
    UHU-menu -- input plugin for freedesktop-style files
    Copyright (C) 2003  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>

#include "uhumenu.h"

static int path_remove;

static struct dirent *readdir_nobackup (DIR *d)
{
	struct dirent *de;
	while ((de = readdir(d)) != NULL) {
		if (de->d_name[0] == '.') continue;
		if (strstr(de->d_name, ".dpkg") != NULL) continue;
		if (strstr(de->d_name, ".rpm") != NULL) continue;
		if (strstr(de->d_name, ".orig") != NULL) continue;
		if (strstr(de->d_name, ".bak") != NULL) continue;
		if (strstr(de->d_name, "~") != NULL) continue;
		return de;
	}
	return NULL;
}

static struct menu *build_menu (const char *dir)
{
	DIR *d;
	struct dirent *de;
	char buf[PATH_MAX];
	struct menu *menu;
	int i;

	d = opendir(dir);
	if (d == NULL) {
		message_e(3, "opendir failed");
		return NULL;
	}
	menu = malloc(sizeof(struct menu));
	menu->nr = 0;
	while ((de = readdir_nobackup(d)) != NULL) menu->nr++;
	menu->items = malloc(menu->nr * sizeof(struct item));
	rewinddir(d);
	i = 0;
	while (i < menu->nr && (de = readdir_nobackup(d)) != NULL) {
		sprintf(buf, "%s/%s", dir, de->d_name);
		if (desktop_parse(buf, path_remove, &menu->items[i]) < 0) continue;
		if (menu->items[i].issubmenu) {
			if ((menu->items[i].submenu = build_menu(buf)) == NULL) {
				continue;
			}
		}
		i++;
	}
	menu->nr = i;
	closedir(d);
	if (i == 0) {
		free(menu->items);
		free(menu);
		return NULL;
	}
	return menu;
}

struct menu *read_menu (const char *path)
{
	char buf[PATH_MAX];
	struct menu *menu;

	sprintf(buf, "%s/menu-source", path);
	path_remove = strlen(buf);

	menu = malloc(sizeof(struct menu));
	menu->nr = 1;
	menu->items = malloc(sizeof(struct item));
	if (desktop_parse(buf, path_remove, &menu->items[0]) < 0) return NULL;
	if (!menu->items[0].issubmenu) {
		message(3, "%s is not a directory", buf);
		return NULL;
	}
	if ((menu->items[0].submenu = build_menu(buf)) == NULL) menu->nr = 0;

	return menu;
}
